@@ -1,6 +1,6 @@
 import csv
 import json
-from datetime import datetime
+from datetime import datetime, date
 from io import StringIO
 from json import JSONEncoder
 from typing import Sequence
@@ -29,6 +29,9 @@ class Encoder(JSONEncoder):
         elif isinstance(o, datetime):
             # print(f'Date encoding')
             return o.isoformat()
+        elif isinstance(o, date):
+            # print(f'Date encoding')
+            return o.isoformat()
         # print(f'Default encoding')
         return JSONEncoder.default(self, o)
 
