@@ -1,10 +1,11 @@
 import re
-from copy import copy, deepcopy
+from copy import deepcopy
 from types import SimpleNamespace
+from typing import Dict, List, Union, Any
 
 from pydantic import BaseModel, Field
-from typhoon.core.dags import IDENTIFIER_REGEX, TaskDefinition, Py, MultiStep, add_yaml_constructors, evaluate_item
-from typing import Dict, List, Union
+
+from typhoon.core.dags import IDENTIFIER_REGEX, TaskDefinition, Py, MultiStep, evaluate_item
 
 
 def task_name(name_in_dag: str, task: str) -> str:
@@ -17,14 +18,14 @@ def config_arg_name(component_name: str, arg: str) -> str:
 
 class ComponentArgument(BaseModel):
     type: str = Field(..., description='Type of your argument')
-    default: Union[str, list, dict, Py] = Field(..., description='Default value for your argument')
+    default: Any = Field(..., description='Default value for your argument')
 
 
 class Component(BaseModel):
     name: str = Field(..., regex=IDENTIFIER_REGEX, description='Name of your DAG')
     args: Dict[str, Union[str, ComponentArgument]]
     tasks: Dict[str, TaskDefinition]
-    output: List[str]
+    output: List[str] = ()
 
     def replace_input_and_args(self, name_in_dag: str, task: TaskDefinition, input_task: str, input_arg_values: dict) -> TaskDefinition:
         if task.input == '$COMPONENT_INPUT':
@@ -90,4 +91,4 @@ class Component(BaseModel):
 
     @property
     def source_tasks(self) -> Dict[str, TaskDefinition]:
-        return {k: v for k, v in self.tasks.items() if v.input is None}
+        return {k: v for k, v in self.tasks.items() if v.input is None or v.input == '$COMPONENT_INPUT'}
