@@ -10,55 +10,46 @@ import yaml
 from pydantic import ValidationError
 from typhoon.core.components import Component
 
-from typhoon.core.dags import DAG, DAGDefinitionV2, add_yaml_constructors
+from typhoon.core.dags import DAGDefinitionV2, add_yaml_constructors
 from typhoon.core.settings import Settings
-from typhoon.core.transpiler import TyphoonFileTemplate
 from typing_extensions import Literal
 
+from typhoon.core.transpiler.dag_transpiler import DagFile
+from typhoon.core.transpiler.task_transpiler import TasksFile
 from typhoon.introspection.introspect_extensions import get_typhoon_extensions_info
 
 
-def transpile_dag_and_store(dag: dict, output_path: Union[str, Path], debug_mode: bool):
-    output_path = Path(output_path)
-    output_path.parent.mkdir(parents=True, exist_ok=True)
-    dag_code = TyphoonFileTemplate(DAG.parse_obj(dag), debug_mode=debug_mode).render()
-    Path(output_path).write_text(dag_code)
+def transpile_dag_and_store(dag: dict, output_folder_path: Union[str, Path], debug_mode: bool):
+    output_folder_path = Path(output_folder_path)
+    output_folder_path.mkdir(parents=True, exist_ok=True)
+    dag = DAGDefinitionV2.parse_obj(dag)
+    dag_code = DagFile(dag, debug_mode=debug_mode).render()
+    (output_folder_path / f'{dag.name}.py').write_text(dag_code)
+    tasks_code = TasksFile(dag.tasks).render()
+    (output_folder_path / 'tasks.py').write_text(tasks_code)
 
 
-def load_dags(ignore_errors: bool = False) -> List[Tuple[DAG, str]]:
-    return [(dd.make_dag(), ds) for dd, ds in load_dag_definitions(ignore_errors)]
-
-
-def load_dag_definitions(ignore_errors: bool = False) -> List[Tuple[DAGDefinitionV2, str]]:
+def load_dag_definitions(ignore_errors: bool = False) -> List[Tuple[DAGDefinitionV2, Path]]:
     add_yaml_constructors()
     dags = []
     for dag_file in Settings.dags_directory.rglob('*.yml'):
-        if ignore_errors:
-            try:
-                dag = DAGDefinitionV2.parse_obj(
-                    yaml.load(dag_file.read_text(), yaml.FullLoader)
-                )
-            except ValidationError:
-                continue
-        else:
+        try:
             dag = DAGDefinitionV2.parse_obj(
-                yaml.safe_load(dag_file.read_text())
+                yaml.load(dag_file.read_text(), yaml.FullLoader)
             )
-        dags.append((dag, dag_file.read_text()))
+        except ValidationError:
+            if ignore_errors:
+                continue
+            else:
+                raise
+        dags.append((dag, dag_file))
 
     return dags
 
 
-def load_dag(dag_name: str, ignore_errors: bool = False) -> Optional[DAG]:
-    dags = load_dags(ignore_errors)
-    matching_dags = [(dag, code) for dag, code in dags if dag.name == dag_name]
-    assert len(matching_dags) <= 1, f'Found {len(matching_dags)} dags with name "{dag_name}"'
-    return matching_dags[0] if len(matching_dags) == 1 else None
-
-
 def load_dag_definition(dag_name: str, ignore_errors: bool = False) -> Optional[DAGDefinitionV2]:
     dags = load_dag_definitions(ignore_errors)
-    matching_dags = [(dag, code) for dag, code in dags if dag.name == dag_name]
+    matching_dags = [dag for dag, _ in dags if dag.name == dag_name]
     assert len(matching_dags) <= 1, f'Found {len(matching_dags)} dags with name "{dag_name}"'
     return matching_dags[0] if len(matching_dags) == 1 else None
 
@@ -70,7 +61,7 @@ def get_dag_errors() -> Dict[str, List[dict]]:
         try:
             DAGDefinitionV2.parse_obj(
                 yaml.load(dag_file.read_text(), yaml.FullLoader)
-            ).make_dag()
+            )
         except ValidationError as e:
             result[dag_file.name.split('.yml')[0]] = e.errors()
 
